package be.iminds.ilabt.jfed;

import be.iminds.ilabt.jfed.fedmon.webapi.client.FedmonWebApiCachedClient;
import be.iminds.ilabt.jfed.fedmon.webapi.client.FedmonWebApiClient;
import be.iminds.ilabt.jfed.fedmon.webapi.client.FedmonWebApiClientConfig;
import be.iminds.ilabt.jfed.fedmon.webapi.client.FedmonWebApiClientPropertiesConfig;
import be.iminds.ilabt.jfed.preferences.CorePreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.JFedTrustStore;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.name.Named;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

/**
 * JFedWebApiClientProvider
 */
public class JFedWebApiClientModule extends AbstractModule {
    public static final String DEFAULT_URL_STRING = "https://flsmonitor-api.fed4fire.eu/";

    @Override
    protected void configure() {

    }

    @Provides
    @Named("webapiUrl")
    @Singleton
    private URL provideWebApiUrl(JFedPreferences jFedPreferences) {
        String userPreference = jFedPreferences.getString(CorePreferenceKey.PREF_JFED_WEBAPI_URL);
        if (userPreference == null || userPreference.trim().isEmpty() ||
                userPreference.trim().equalsIgnoreCase("BUILTIN") || userPreference.trim().equalsIgnoreCase("DEFAULT")) {
            try {
                return new URL(DEFAULT_URL_STRING);
            } catch (MalformedURLException e) {
                throw new RuntimeException("Bug: builtin URL is invalid: \""+DEFAULT_URL_STRING+"\"");
            }
        } else {
            try {
                return new URL(userPreference);
            } catch (MalformedURLException e) {
                throw new RuntimeException("jFed config option \""+CorePreferenceKey.PREF_JFED_WEBAPI_URL.getKey()+"\" has an invalid value: \""+userPreference+"\"", e);
            }
        }
    }

    @Provides
    @Singleton
    private FedmonWebApiClient provideFedmonWebApiClient(JFedTrustStore trustStore,
                                                         @Named("webapiUrl") URL url,
                                                         be.iminds.ilabt.jfed.log.Logger logger) {
        Properties fedmonWebApiClientConfigProperties = new Properties();
        fedmonWebApiClientConfigProperties.setProperty("webapi_client_url_read_base", url.toExternalForm());
        fedmonWebApiClientConfigProperties.setProperty("webapi_client_url_write_base", "http://localhost/"); //no need ever to write

        FedmonWebApiClientConfig fedmonWebApiClientConfig = new FedmonWebApiClientPropertiesConfig(
                fedmonWebApiClientConfigProperties, trustStore, null/*userCertificate*/, null/*userPrivateKey*/, logger);

        FedmonWebApiClient fedmonWebApiClient = new FedmonWebApiCachedClient(fedmonWebApiClientConfig);
        return fedmonWebApiClient;
    }
}
