package be.iminds.ilabt.jfed.experimenter_gui.bugreporting;

import be.iminds.ilabt.jfed.CorePreferencesModule;
import be.iminds.ilabt.jfed.experimenter_gui.JFedVersionInfo;
import be.iminds.ilabt.jfed.experimenter_gui.connectivity_tester.ConnectivityTester;
import be.iminds.ilabt.jfed.experimenter_gui.util.BugReportExperimentControllerListener;
import be.iminds.ilabt.jfed.highlevel.HighLevelModule;
import be.iminds.ilabt.jfed.highlevel.LowLevelModule;
import be.iminds.ilabt.jfed.highlevel.MockGeniUserModule;
import be.iminds.ilabt.jfed.highlevel.SfaOnlyExperimentModule;
import be.iminds.ilabt.jfed.highlevel.controller.JavaFXTaskThread;
import be.iminds.ilabt.jfed.highlevel.model.SfaModel;
import be.iminds.ilabt.jfed.log.cache.AllApiCallsList;
import be.iminds.ilabt.jfed.log.cache.ApiCallDetailsCache;
import be.iminds.ilabt.jfed.logging.JFedLoggerManager;
import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import javafx.collections.FXCollections;
import org.testng.annotations.Guice;
import org.testng.annotations.Test;

import javax.inject.Inject;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.StringReader;
import java.util.Collections;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;


@Test

@Guice(modules = {HighLevelModule.class, LowLevelModule.class, MockGeniUserModule.class, CorePreferencesModule.class, SfaOnlyExperimentModule.class})
public class BugReportComposerTest {

    @Inject
    private BugReportReaderFactory bugReportReaderFactory;
    @Inject
    private ApiCallDetailsCache cache;
    @Inject
    private JFedVersionInfo jFedVersionInfo;

    //TODO: make a reliable test for StitchingJobReport
    @Test(enabled = false)
    public void testStitchingReports() throws Exception {
        JavaFXTaskThread javaFXTaskThread = mock(JavaFXTaskThread.class);
        GeniUserProvider userLoginModelManager = mock(GeniUserProvider.class);
        SfaModel sfaModel = mock(SfaModel.class);
        ConnectivityTester connectivityTester = mock(ConnectivityTester.class);
        JFedGuiPreferences jFedPreferences = mock(JFedGuiPreferences.class);
        JFedLoggerManager jFedLoggerManager = mock(JFedLoggerManager.class);
        BugReportExperimentControllerListener bugReportExperimentControllerListener =
                mock(BugReportExperimentControllerListener.class);
        AllApiCallsList allApiCallsList = mock(AllApiCallsList.class);

        GeniUser geniUser = mock(GeniUser.class);
        when(geniUser.getUserUrnString()).thenReturn("testuserurn");

        when(userLoginModelManager.getLoggedInGeniUser()).thenReturn(geniUser);

        when(connectivityTester.getTestsFinished()).thenReturn(true);

        when(jFedLoggerManager.getBugReportMemoryAppender()).thenReturn(null);
        when(javaFXTaskThread.getCallsCount()).thenReturn(0);
        when(connectivityTester.getTestTasks()).thenReturn(FXCollections.emptyObservableList());
        when(jFedPreferences.getPropertiesFile()).thenReturn(File.createTempFile("empty", ".properties"));
        when(javaFXTaskThread.getAllTasks()).thenReturn(FXCollections.emptyObservableList());
        when(sfaModel.getSlices()).thenReturn(FXCollections.emptyObservableList());

        SerializableJobReport jobReport = new SerializableJobReport();

        when(bugReportExperimentControllerListener.getStitchingJobReports()).thenReturn(Collections.EMPTY_LIST);

        when(allApiCallsList.getCalls()).thenReturn(Collections.emptyList());

        BugReportComposer bugReportComposer = new BugReportComposer(
                javaFXTaskThread,
                userLoginModelManager,
                sfaModel,
                connectivityTester,
                jFedPreferences,
                jFedLoggerManager,
                bugReportExperimentControllerListener,
                jFedVersionInfo,
                allApiCallsList);
        bugReportComposer.setDescription("Test Description");
        bugReportComposer.setEmail("Test Email");

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        bugReportComposer.writeToOutputStream(baos);
        baos.close();
        String xml = new String(baos.toByteArray(), "UTF-8");
        System.out.println("Created BugReport xml: " + xml);

        //just a quick check to see if XML looks OK
        assertThat(xml, containsString("stitchingreports"));
        assertThat(xml, containsString("Still OK 3"));

        //now reconstruct
        BugReportReader reader = bugReportReaderFactory.createBugReportReader(new StringReader(xml));

        BugReport bugReport = reader.call();

        assertThat(bugReport, is(notNullValue()));
     //   assertThat(bugReport.getStitchingReports(), hasSize(1));
       // StitchingReport reconstructedReport = bugReport.getStitchingReports().get(0);
//        assertThat(reconstructedReport, is(notNullValue()));
//        assertThat(reconstructedReport.getReportStates(), hasSize(3));
//        assertThat(reconstructedReport.getLastState("First").getCalls(), hasSize(0));
//        assertThat(reconstructedReport.getLastState("First").getLogLines(), hasSize(1));
//        assertThat(reconstructedReport.getLastState("First").getLogLines().get(0).getType(), equalTo(LogOutput.LogLineType.DEBUG));
//        assertThat(reconstructedReport.getLastState("First").getLogLines().get(0).getText(), equalTo("Testing 1"));
//        assertThat(reconstructedReport.getLastState("Second").getLogLines(), hasSize(1));
//        assertThat(reconstructedReport.getLastState("Second").getLogLines().get(0).getText(), equalTo("OK 2"));
//        assertThat(reconstructedReport.getLastState("Second").getLogLines().get(0).getType(), equalTo(LogOutput.LogLineType.WARN));
//        assertThat(reconstructedReport.getLastState("Third").getLogLines(), hasSize(1));
//        assertThat(reconstructedReport.getLastState("Third").getLogLines().get(0).getText(), equalTo("Still OK 3"));
//        assertThat(reconstructedReport.getLastState("Third").getLogLines().get(0).getType(), equalTo(LogOutput.LogLineType.ERROR));
    }
}
